/*
  This file is part of TALER
  Copyright (C) 2014-2026 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2.1, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License along with
  TALER; see the file COPYING.LGPL.  If not, see
  <http://www.gnu.org/licenses/>
*/
/**
 * @file src/lib/merchant_api_get-management-instances.c
 * @brief Implementation of the GET /management/instances request
 * @author Christian Grothoff
 */
#include "platform.h"
#include <curl/curl.h>
#include <jansson.h>
#include <microhttpd.h> /* just for HTTP status codes */
#include <gnunet/gnunet_util_lib.h>
#include <gnunet/gnunet_curl_lib.h>
#include <taler/merchant/get-management-instances.h>
#include "merchant_api_curl_defaults.h"
#include <taler/taler_json_lib.h>

/**
 * Maximum number of instances permitted.
 */
#define MAX_INSTANCES 1024


/**
 * Handle for a GET /management/instances operation.
 */
struct TALER_MERCHANT_GetManagementInstancesHandle
{
  /**
   * Base URL of the merchant backend.
   */
  char *base_url;

  /**
   * The full URL for this request.
   */
  char *url;

  /**
   * Handle for the request.
   */
  struct GNUNET_CURL_Job *job;

  /**
   * Function to call with the result.
   */
  TALER_MERCHANT_GetManagementInstancesCallback cb;

  /**
   * Closure for @a cb.
   */
  TALER_MERCHANT_GET_MANAGEMENT_INSTANCES_RESULT_CLOSURE *cb_cls;

  /**
   * Reference to the execution context.
   */
  struct GNUNET_CURL_Context *ctx;
};


/**
 * Parse instance information from @a ia.
 *
 * @param ia JSON array (or NULL!) with instance data
 * @param[in] igr partially filled response
 * @param gimh operation handle
 * @return #GNUNET_OK on success
 */
static enum GNUNET_GenericReturnValue
parse_instances (const json_t *ia,
                 struct TALER_MERCHANT_GetManagementInstancesResponse *igr,
                 struct TALER_MERCHANT_GetManagementInstancesHandle *gimh)
{
  unsigned int iis_len = (unsigned int) json_array_size (ia);

  if ( (json_array_size (ia) != (size_t) iis_len) ||
       (iis_len > MAX_INSTANCES) )
  {
    GNUNET_break (0);
    return GNUNET_SYSERR;
  }
  {
    struct TALER_MERCHANT_GetManagementInstancesInstanceInfo iis[
      GNUNET_NZL (iis_len)];
    size_t index;
    json_t *value;

    memset (iis,
            0,
            sizeof (iis));
    json_array_foreach (ia, index, value) {
      struct TALER_MERCHANT_GetManagementInstancesInstanceInfo *ii =
        &iis[index];
      struct GNUNET_JSON_Specification spec[] = {
        GNUNET_JSON_spec_string ("name",
                                 &ii->name),
        TALER_JSON_spec_slug ("id",
                              &ii->id),
        GNUNET_JSON_spec_fixed_auto ("merchant_pub",
                                     &ii->merchant_pub),
        GNUNET_JSON_spec_array_const ("payment_targets",
                                      &ii->payment_targets),
        GNUNET_JSON_spec_mark_optional (
          TALER_JSON_spec_web_url ("website",
                                   &ii->website),
          NULL),
        GNUNET_JSON_spec_mark_optional (
          GNUNET_JSON_spec_string ("logo",
                                   &ii->logo),
          NULL),
        GNUNET_JSON_spec_bool ("deleted",
                               &ii->deleted),
        GNUNET_JSON_spec_end ()
      };

      if (GNUNET_OK !=
          GNUNET_JSON_parse (value,
                             spec,
                             NULL, NULL))
      {
        GNUNET_break_op (0);
        return GNUNET_SYSERR;
      }
      for (size_t i = 0; i<json_array_size (ii->payment_targets); i++)
      {
        if  (! json_is_string (json_array_get (ii->payment_targets,
                                               i)))
        {
          GNUNET_break_op (0);
          return GNUNET_SYSERR;
        }
      }
    }
    igr->details.ok.iis_length = iis_len;
    igr->details.ok.iis = iis;
    gimh->cb (gimh->cb_cls,
              igr);
    gimh->cb = NULL;
  }
  return GNUNET_OK;
}


/**
 * Function called when we're done processing the
 * HTTP GET /management/instances request.
 *
 * @param cls the `struct TALER_MERCHANT_GetManagementInstancesHandle`
 * @param response_code HTTP response code, 0 on error
 * @param response response body, NULL if not in JSON
 */
static void
handle_get_instances_finished (void *cls,
                               long response_code,
                               const void *response)
{
  struct TALER_MERCHANT_GetManagementInstancesHandle *gimh = cls;
  const json_t *json = response;
  struct TALER_MERCHANT_GetManagementInstancesResponse igr = {
    .hr.http_status = (unsigned int) response_code,
    .hr.reply = json
  };

  gimh->job = NULL;
  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
              "Got /management/instances response with status code %u\n",
              (unsigned int) response_code);
  switch (response_code)
  {
  case MHD_HTTP_OK:
    {
      const json_t *instances;
      struct GNUNET_JSON_Specification spec[] = {
        GNUNET_JSON_spec_array_const ("instances",
                                      &instances),
        GNUNET_JSON_spec_end ()
      };

      if (GNUNET_OK !=
          GNUNET_JSON_parse (json,
                             spec,
                             NULL, NULL))
      {
        igr.hr.http_status = 0;
        igr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
        break;
      }
      if (GNUNET_OK ==
          parse_instances (instances,
                           &igr,
                           gimh))
      {
        TALER_MERCHANT_get_management_instances_cancel (gimh);
        return;
      }
      igr.hr.http_status = 0;
      igr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
      break;
    }
  case MHD_HTTP_UNAUTHORIZED:
    igr.hr.ec = TALER_JSON_get_error_code (json);
    igr.hr.hint = TALER_JSON_get_error_hint (json);
    break;
  default:
    igr.hr.ec = TALER_JSON_get_error_code (json);
    igr.hr.hint = TALER_JSON_get_error_hint (json);
    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                "Unexpected response code %u/%d\n",
                (unsigned int) response_code,
                (int) igr.hr.ec);
    break;
  }
  gimh->cb (gimh->cb_cls,
            &igr);
  TALER_MERCHANT_get_management_instances_cancel (gimh);
}


struct TALER_MERCHANT_GetManagementInstancesHandle *
TALER_MERCHANT_get_management_instances_create (
  struct GNUNET_CURL_Context *ctx,
  const char *url)
{
  struct TALER_MERCHANT_GetManagementInstancesHandle *gimh;

  gimh = GNUNET_new (struct TALER_MERCHANT_GetManagementInstancesHandle);
  gimh->ctx = ctx;
  gimh->base_url = GNUNET_strdup (url);
  return gimh;
}


enum TALER_ErrorCode
TALER_MERCHANT_get_management_instances_start (
  struct TALER_MERCHANT_GetManagementInstancesHandle *gimh,
  TALER_MERCHANT_GetManagementInstancesCallback cb,
  TALER_MERCHANT_GET_MANAGEMENT_INSTANCES_RESULT_CLOSURE *cb_cls)
{
  CURL *eh;

  gimh->cb = cb;
  gimh->cb_cls = cb_cls;
  gimh->url = TALER_url_join (gimh->base_url,
                              "management/instances",
                              NULL);
  if (NULL == gimh->url)
    return TALER_EC_GENERIC_CONFIGURATION_INVALID;
  eh = TALER_MERCHANT_curl_easy_get_ (gimh->url);
  if (NULL == eh)
    return TALER_EC_GENERIC_CONFIGURATION_INVALID;
  gimh->job = GNUNET_CURL_job_add (gimh->ctx,
                                   eh,
                                   &handle_get_instances_finished,
                                   gimh);
  if (NULL == gimh->job)
    return TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE;
  return TALER_EC_NONE;
}


void
TALER_MERCHANT_get_management_instances_cancel (
  struct TALER_MERCHANT_GetManagementInstancesHandle *gimh)
{
  if (NULL != gimh->job)
  {
    GNUNET_CURL_job_cancel (gimh->job);
    gimh->job = NULL;
  }
  GNUNET_free (gimh->url);
  GNUNET_free (gimh->base_url);
  GNUNET_free (gimh);
}


/* end of merchant_api_get-management-instances-new.c */
